@@ -2,26 +2,29 @@
 
 // open up changes.jpg, stretch it to fill, drop it to greyscale
 static int
-draw_background(struct notcurses* nc){
-  if(notcurses_canopen_images(nc)){
-    struct ncplane* n = notcurses_stdplane(nc);
-    char* path = find_data("changes.jpg");
-    struct ncvisual* ncv = ncvisual_from_file(path);
-    free(path);
-    if(!ncv){
-      return -1;
-    }
-    struct ncvisual_options vopts = {
-      .scaling = NCSCALE_STRETCH,
-      .n = n,
-    };
-    if(ncvisual_blit(nc, ncv, &vopts) == NULL){
-      ncvisual_destroy(ncv);
-      return -1;
-    }
-    ncplane_greyscale(n);
-    ncvisual_destroy(ncv);
+draw_background(struct notcurses* nc, struct ncplane** bgp){
+  *bgp = NULL;
+  if(!notcurses_canopen_images(nc)){
+    return 0;
   }
+  struct ncplane* n = notcurses_stdplane(nc);
+  char* path = find_data("changes.jpg");
+  struct ncvisual* ncv = ncvisual_from_file(path);
+  free(path);
+  if(!ncv){
+    return -1;
+  }
+  struct ncvisual_options vopts = {
+    .scaling = NCSCALE_STRETCH,
+    .n = n,
+    .flags = NCVISUAL_OPTION_CHILDPLANE,
+  };
+  if((*bgp = ncvisual_blit(nc, ncv, &vopts)) == NULL){
+    ncvisual_destroy(ncv);
+    return -1;
+  }
+  ncplane_greyscale(n);
+  ncvisual_destroy(ncv);
   return 0;
 }
 
@@ -233,7 +236,8 @@ selector_run(struct notcurses* nc, struct ncplane* p, struct ncselector* selecto
     "NCSelector allows a single option to be selected from a list.\n"
     "NCFdplane streams a file descriptor, while NCSubproc spawns a subprocess and streams its output.\n";
   int titers = get_word_count(text);
-  int ret = 0, dimy, dimx;
+  int ret = 0;
+  unsigned dimy, dimx;
   ncplane_dim_yx(notcurses_stdplane(nc), &dimy, &dimx);
   const int centery = (dimy - ncplane_dim_y(p)) / 2;
   int ry, rx, sy, sx;
@@ -307,7 +311,8 @@ mselector_run(struct notcurses* nc, struct ncplane* p, struct ncmultiselector* m
     "Widgets can be controlled with the keyboard and/or mouse. "
     "They are implemented atop ncplanes, and these planes can be manipulated like all others.";
   const int titers = get_word_count(text);
-  int ret = 0, dimy, dimx;
+  int ret = 0;
+  unsigned dimy, dimx;
   ncplane_dim_yx(notcurses_stdplane(nc), &dimy, &dimx);
   const int centery = (dimy - ncplane_dim_y(p)) / 2;
   int ry, rx, sy, sx;
@@ -376,7 +381,7 @@ mselector_run(struct notcurses* nc, struct ncplane* p, struct ncmultiselector* m
 static int
 reader_demo(struct notcurses* nc){
   int ret = -1;
-  int dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* std = notcurses_stddim_yx(nc, &dimy, &dimx);
   const int READER_COLS = 64;
   const int READER_ROWS = 8;
@@ -434,9 +439,13 @@ done:
 // a plane with exposition text rises from the bottom to the center of the
 // screen. as it does so, two widgets (selector and multiselector) come in
 // from the left and right, respectively. they then fade out.
-int zoo_demo(struct notcurses* nc){
-  if(draw_background(nc)){
+int zoo_demo(struct notcurses* nc, uint64_t startns){
+  (void)startns;
+  struct ncplane* bgp;
+  if(draw_background(nc, &bgp)){
     return -1;
   }
-  return reader_demo(nc);
+  int ret = reader_demo(nc);
+  ncplane_destroy(bgp);
+  return ret;
 }
