@@ -125,7 +125,7 @@ const char* nckeystr(char32_t spkey){
     case NCKEY_EXIT:    return "exit";
     case NCKEY_PRINT:   return "print";
     case NCKEY_REFRESH: return "refresh";
-    case NCKEY_SEPARATOR: return "seperator";
+    case NCKEY_SEPARATOR: return "separator";
     case NCKEY_CAPS_LOCK: return "caps lock";
     case NCKEY_SCROLL_LOCK: return "scroll lock";
     case NCKEY_NUM_LOCK: return "num lock";
@@ -310,13 +310,13 @@ int input_demo(ncpp::NotCurses* nc) {
     if(r == 0){ // interrupted by signal
       continue;
     }
-    if(((r == 'D' || r == 'd') && ni.ctrl) || r == NCKEY_EOF){
+    if((r == 'D' && ncinput_ctrl_p(&ni)) || r == NCKEY_EOF){
       done = true;
       tid.join();
       ncuplot_destroy(plot);
       return 0;
     }
-    if((r == 'L' || r == 'l') && ni.ctrl){
+    if(r == 'L' && ncinput_ctrl_p(&ni)){
       mtx.lock();
         if(!nc->refresh(nullptr, nullptr)){
           mtx.unlock();
@@ -328,8 +328,16 @@ int input_demo(ncpp::NotCurses* nc) {
       break;
     }
     n->set_fg_rgb8(0xd0, 0xd0, 0xd0);
-    n->printf("%c%c%c%c ", ni.alt ? 'A' : 'a', ni.ctrl ? 'C' : 'c',
-              ni.shift ? 'S' : 's', evtype_to_char(&ni));
+    n->printf("%c%c%c%c%c%c%c%c%c ",
+              ncinput_shift_p(&ni) ? 'S' : 's',
+              ncinput_alt_p(&ni) ? 'A' : 'a',
+              ncinput_ctrl_p(&ni) ? 'C' : 'c',
+              ncinput_super_p(&ni) ? 'U' : 'u',
+              ncinput_hyper_p(&ni) ? 'H' : 'h',
+              ncinput_meta_p(&ni) ? 'M' : 'm',
+              ncinput_capslock_p(&ni) ? 'X' : 'x',
+              ncinput_numlock_p(&ni) ? '#' : '.',
+              evtype_to_char(&ni));
     if(r < 0x80){
       n->set_fg_rgb8(128, 250, 64);
       if(n->printf("ASCII: [0x%02x (%03d)] '%lc'", r, r,
@@ -343,7 +351,7 @@ int input_demo(ncpp::NotCurses* nc) {
           break;
         }
         if(NCKey::IsMouse(r)){
-          if(n->printf(-1, NCAlign::Right, " x: %d y: %d", ni.x, ni.y) < 0){
+          if(n->printf(-1, NCAlign::Right, " %d/%d", ni.x, ni.y) < 0){
             break;
           }
         }
