@@ -200,7 +200,10 @@ cursor_yx_get(ncdirect* n, const char* u7, unsigned* y, unsigned* x){
   if(x == NULL){
     x = &fakex;
   }
-  get_cursor_location(ictx, u7, y, x);
+  if(get_cursor_location(ictx, u7, y, x)){
+    logerror("couldn't get cursor position");
+    return -1;
+  }
   loginfo("cursor at y=%u x=%u\n", *y, *x);
   return 0;
 }
@@ -632,7 +635,7 @@ int ncdirect_raster_frame(ncdirect* n, ncdirectv* ncdv, ncalign_e align){
 static ncdirectv*
 ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
                        const struct ncvisual_options* vopts){
-  struct ncvisual_options defvopts = {};
+  struct ncvisual_options defvopts = {0};
   if(!vopts){
     vopts = &defvopts;
   }
@@ -711,7 +714,7 @@ ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
     free_plane(ncdv);
     return NULL;
   }
-  blitterargs bargs = {};
+  blitterargs bargs = {0};
   bargs.flags = vopts->flags;
   if(vopts->flags & NCVISUAL_OPTION_ADDALPHA){
     bargs.transcolor = vopts->transcolor | 0x1000000ull;
@@ -743,7 +746,7 @@ ncdirectv* ncdirect_render_frame(ncdirect* n, const char* file,
   if(ncv == NULL){
     return NULL;
   }
-  struct ncvisual_options vopts = {};
+  struct ncvisual_options vopts = {0};
   const struct blitset* bset = rgba_blitter_low(&n->tcache, scale, true, blitfxn);
   if(!bset){
     return NULL;
@@ -832,7 +835,7 @@ static int
 ncdirect_stop_minimal(void* vnc){
   ncdirect* nc = vnc;
   int ret = drop_signals(nc);
-  fbuf f = {};
+  fbuf f = {0};
   if(fbuf_init_small(&f) == 0){
     ret |= reset_term_attributes(&nc->tcache, &f);
     ret |= fbuf_finalize(&f, stdout);
@@ -856,7 +859,7 @@ ncdirect_stop_minimal(void* vnc){
     ret |= tcsetattr(nc->tcache.ttyfd, TCSANOW, nc->tcache.tpreserved);
   }
   ret |= ncdirect_flush(nc);
-#ifndef __MINGW64__
+#ifndef __MINGW32__
   del_curterm(cur_term);
 #endif
   return ret;
@@ -973,7 +976,11 @@ int ncdirect_stop(ncdirect* nc){
 char* ncdirect_readline(ncdirect* n, const char* prompt){
   const char* u7 = get_escape(&n->tcache, ESCAPE_U7);
   if(!u7){ // we probably *can*, but it would be a pita; screw it
-    logerror("can't readline without u7\n");
+    logerror("can't readline without u7");
+    return NULL;
+  }
+  if(n->eof){
+    logerror("already got EOF");
     return NULL;
   }
   if(fprintf(n->ttyfp, "%s", prompt) < 0){
@@ -1005,15 +1012,18 @@ char* ncdirect_readline(ncdirect* n, const char* prompt){
     if(ni.evtype == NCTYPE_RELEASE){
       continue;
     }
-    if(id == NCKEY_EOF || id == NCKEY_ENTER){
+    if(id == NCKEY_EOF || id == NCKEY_ENTER || (ncinput_ctrl_p(&ni) && id == 'D')){
       if(id == NCKEY_ENTER){
         if(fputc('\n', n->ttyfp) < 0){
           free(str);
           return NULL;
         }
-      }else if(wused == 1){ // NCKEY_EOF without input returns NULL
-        free(str);
-        return NULL;
+      }else{
+        n->eof = 1;
+        if(wused == 1){ // NCKEY_EOF without input returns NULL
+          free(str);
+          return NULL;
+        }
       }
       char* ustr = ncwcsrtombs(str);
       free(str);
@@ -1032,9 +1042,9 @@ char* ncdirect_readline(ncdirect* n, const char* prompt){
       wpos -= dimx;
     }else if(id == NCKEY_DOWN){
       wpos += dimx;
-    }else if(id == 'a' && ni.ctrl){
+    }else if(id == 'A' && ncinput_ctrl_p(&ni)){
       wpos = 1;
-    }else if(id == 'e' && ni.ctrl){
+    }else if(id == 'E' && ncinput_ctrl_p(&ni)){
       wpos = wused - 1;
     }else if(nckey_synthesized_p(ni.id)){
       continue;
@@ -1148,7 +1158,7 @@ int ncdirect_on_styles(ncdirect* n, unsigned stylebits){
     return -1;
   }
   uint32_t stylemask = n->stylemask | stylebits;
-  fbuf f = {};
+  fbuf f = {0};
   if(fbuf_init_small(&f)){
     return -1;
   }
@@ -1169,7 +1179,7 @@ uint16_t ncdirect_styles(const ncdirect* n){
 // turn off any specified stylebits
 int ncdirect_off_styles(ncdirect* n, unsigned stylebits){
   uint32_t stylemask = n->stylemask & ~stylebits;
-  fbuf f = {};
+  fbuf f = {0};
   if(fbuf_init_small(&f)){
     return -1;
   }
@@ -1189,7 +1199,7 @@ int ncdirect_set_styles(ncdirect* n, unsigned stylebits){
     return -1;
   }
   uint32_t stylemask = stylebits;
-  fbuf f = {};
+  fbuf f = {0};
   if(fbuf_init_small(&f)){
     return -1;
   }
@@ -1398,7 +1408,7 @@ int ncdirect_box(ncdirect* n, uint64_t ul, uint64_t ur,
   }else{
     ncdirect_cursor_right(n, 1);
   }
-  mbstate_t ps = {};
+  mbstate_t ps = {0};
   size_t bytes;
   if((bytes = wcrtomb(hl, wchars[4], &ps)) == (size_t)-1){
     logerror("error converting %lc\n", wchars[4]);
@@ -1552,7 +1562,7 @@ int ncdirect_stream(ncdirect* n, const char* filename, ncstreamcb streamer,
     }
     if(lastid > -1){
       if(n->tcache.pixel_remove){
-        fbuf f = {};
+        fbuf f = {0};
         fbuf_init_small(&f);
         if(n->tcache.pixel_remove(lastid, &f)){
           fbuf_free(&f);
