@@ -182,7 +182,7 @@ fbuf_puts(fbuf* f, const char* s){
 
 static inline int
 fbuf_putint(fbuf* f, int n){
-  if(fbuf_grow(f, 10)){ // 32-bit int might require up to 10 digits
+  if(fbuf_grow(f, 20)){ // 64-bit int might require up to 20 digits
     return -1;
   }
   uint64_t r = snprintf(f->buf + f->used, f->size - f->used, "%d", n);
@@ -194,6 +194,20 @@ fbuf_putint(fbuf* f, int n){
   return r;
 }
 
+static inline int
+fbuf_putuint(fbuf* f, int n){
+  if(fbuf_grow(f, 20)){ // 64-bit int might require up to 20 digits
+    return -1;
+  }
+  uint64_t r = snprintf(f->buf + f->used, f->size - f->used, "%u", n);
+  if(r > f->size - f->used){
+    assert(r <= f->size - f->used);
+    return -1; // FIXME grow?
+  }
+  f->used += r;
+  return r;
+}
+
 // FIXME eliminate this, ideally
 __attribute__ ((format (printf, 2, 3)))
 static inline int
@@ -229,11 +243,11 @@ fbuf_emit(fbuf* f, const char* esc){
 static inline void
 fbuf_free(fbuf* f){
   if(f){
-//    logdebug("Releasing from %" PRIu32 "B (%" PRIu32 "B)\n", f->size, f->used);
+//    logdebug("Releasing from %" PRIu32 "B (%" PRIu32 "B)", f->size, f->used);
     if(f->buf){
 #if __linux__
       if(munmap(f->buf, f->size)){
-        //logwarn("Error unmapping alloc (%s)\n", strerror(errno));
+        //logwarn("Error unmapping alloc (%s)", strerror(errno));
       }
 #else
       free(f->buf);
@@ -255,14 +269,14 @@ blocking_write(int fd, const char* buf, size_t buflen){
     ssize_t w = write(fd, buf + written, buflen - written);
     if(w < 0){
       if(errno != EAGAIN && errno != EWOULDBLOCK && errno != EINTR && errno != EBUSY){
-        logerror("Error writing out data on %d (%s)\n", fd, strerror(errno));
+        logerror("Error writing out data on %d (%s)", fd, strerror(errno));
         return -1;
       }
     }else{
       written += w;
     }
     // FIXME ought probably use WSAPoll() on windows
-#ifndef __MINGW64__
+#ifndef __MINGW32__
     if(written < buflen){
       struct pollfd pfd = {
         .fd = fd,
