@@ -66,6 +66,7 @@ struct initial_responses {
   bool got_bg;                 // have we read default background?
   bool got_fg;                 // have we read default foreground?
   bool rgb;                    // was RGB DirectColor advertised?
+  bool rectangular_edits;      // were rectangular edits advertised?
   int pixx;                    // screen geometry in pixels
   int pixy;                    // screen geometry in pixels
   int dimx;                    // screen geometry in cells
@@ -80,6 +81,8 @@ struct initial_responses {
   unsigned kbdlevel;           // enabled kitty keyboard functions
   ncpalette palette;           // palette entries
   int maxpaletteread;          // maximum palette index read
+  bool pixelmice;              // have we pixel-based mice events?
+  char* hpa;                   // control sequence for hpa via XTGETTCAP
 };
 
 // Blocking call. Waits until the input thread has processed all responses to
