@@ -12,7 +12,7 @@ extern "C" {
 #include "notcurses/notcurses.h"
 #include "notcurses/direct.h"
 
-#ifndef __MINGW64__
+#ifndef __MINGW32__
 #define API __attribute__((visibility("default")))
 #else
 #define API __declspec(dllexport)
@@ -37,7 +37,7 @@ extern "C" {
 #include <stdbool.h>
 #include <inttypes.h>
 #include <unictype.h>
-#ifndef __MINGW64__
+#ifndef __MINGW32__
 #include <langinfo.h>
 #endif
 #include "lib/termdesc.h"
@@ -252,15 +252,18 @@ typedef struct ncdirect {
   uint16_t stylemask;        // current styles
   uint64_t flags;            // copied in ncdirect_init() from param
   ncsharedstats stats;       // stats! not as broadly used as in notcurses
+  unsigned eof;              // have we seen EOF on stdin?
 } ncdirect;
 
 // Extracellular state for a cell during the render process. There is one
-// crender per rendered cell, and they are initialized to all zeroes.
+// crender per rendered cell, and they are initialized to all zeroes. Be
+// careful with order here; padding can easily enlarge this struct from 40
+// to 48 bytes.
 struct crender {
+  nccell c;         // solution cell
   const ncplane *p; // source of glyph for this cell
-  nccell c;
-  uint32_t hcfg;       // fg channel prior to HIGHCONTRAST (need full channel)
-  sprixel* sprixel;    // bitmap encountered during traversal
+  sprixel* sprixel; // bitmap encountered during traversal
+  uint32_t hcfg;    // fg channel prior to HIGHCONTRAST (need full channel)
   struct {
     // If the glyph we render is from an ncvisual, and has a transparent or
     // blended background, blitter stacking is in effect. This is a complicated
@@ -388,12 +391,6 @@ typedef struct blitterargs {
   } u;
 } blitterargs;
 
-// scaledy and scaledx are output geometry from scaling; data is output data
-// from scaling. we might actually need more pixels due to framing concerns,
-// in which case just assume transparent input pixels where needed.
-typedef int (*ncblitter)(struct ncplane* n, int linesize, const void* data,
-                         int scaledy, int scaledx, const blitterargs* bargs);
-
 // a system for rendering RGBA pixels as text glyphs or sixel/kitty bitmaps
 struct blitset {
   ncblitter_e geom;
@@ -461,7 +458,7 @@ static inline int
 ncplane_set_widget(ncplane* n, void* w, void(*wdestruct)(void*)){
   if(n->widget){
     if(w){
-      logerror("plane is already bound to a widget\n");
+      logerror("plane is already bound to a widget");
       return -1;
     }
   }else if(w == NULL){
@@ -646,8 +643,10 @@ ncpile_debug(const ncpile* p, fbuf* f){
     fbuf_printf(f, "%04d off y: %3d x: %3d geom y: %3u x: %3u curs y: %3u x: %3u %p %.4s\n",
                 planeidx, n->absy, n->absx, n->leny, n->lenx, n->y, n->x, n, n->name);
     if(n->boundto || n->bnext || n->bprev || n->blist){
-      fbuf_printf(f, " bound %p ← %p → %p binds %p\n",
-                  n->boundto, n->bprev, n->bnext, n->blist);
+      fbuf_printf(f, " bound %p %s %p %s %p binds %p\n",
+                  n->boundto, notcurses_canutf8(ncpile_notcurses_const(p)) ? "←" : "<",
+                  n->bprev, notcurses_canutf8(ncpile_notcurses_const(p)) ? "→" : ">",
+                  n->bnext, n->blist);
     }
     if(n->bprev && (*n->bprev != n)){
       fbuf_printf(f, " WARNING: expected *->bprev %p, got %p\n", n, *n->bprev);
@@ -691,7 +690,6 @@ void sprixel_hide(sprixel* s);
 // dimy and dimx are cell geometry, not pixel.
 sprixel* sprixel_alloc(ncplane* n, int dimy, int dimx);
 sprixel* sprixel_recycle(ncplane* n);
-int sprite_init(const tinfo* t, int fd);
 int sprite_clear_all(const tinfo* t, fbuf* f);
 // these three all use absolute coordinates
 void sprixel_invalidate(sprixel* s, int y, int x);
@@ -706,7 +704,7 @@ void sixelmap_free(struct sixelmap *s);
 static inline int
 sprite_scrub(const notcurses* n, const ncpile* p, sprixel* s){
 //sprixel_debug(s, stderr);
-  logdebug("sprixel %u state %d\n", s->id, s->invalidated);
+  logdebug("sprixel %u state %d", s->id, s->invalidated);
   return n->tcache.pixel_scrub(p, s);
 }
 
@@ -719,7 +717,7 @@ sprite_draw(const tinfo* ti, const ncpile* p, sprixel* s, fbuf* f,
     return 0;
   }
 //sprixel_debug(s, stderr);
-  logdebug("sprixel %u state %d\n", s->id, s->invalidated);
+  logdebug("sprixel %u state %d", s->id, s->invalidated);
   return ti->pixel_draw(ti, p, s, f, yoff, xoff);
 }
 
@@ -729,7 +727,7 @@ static inline int
 sprite_redraw(notcurses* nc, const ncpile* p, sprixel* s, fbuf* f, int y, int x){
 //sprixel_debug(s, stderr);
   const tinfo* ti = &nc->tcache;
-  logdebug("sprixel %u state %d\n", s->id, s->invalidated);
+  logdebug("sprixel %u state %d", s->id, s->invalidated);
   if(s->invalidated == SPRIXEL_MOVED && ti->pixel_move){
     // if we are kitty prior to 0.20.0, C=1 isn't available to us, and we must
     // not emit it. we use sixel_maxy_pristine as a side channel to encode
@@ -785,7 +783,7 @@ destroy_tam(ncplane* p){
 
 static inline int
 sprite_rebuild(const notcurses* nc, sprixel* s, int ycell, int xcell){
-  logdebug("rebuilding %d %d/%d\n", s->id, ycell, xcell);
+  logdebug("rebuilding %d %d/%d", s->id, ycell, xcell);
   const int idx = s->dimx * ycell + xcell;
   int ret = 0;
   // special case the transition back to SPRIXCELL_TRANSPARENT; this can be
@@ -879,7 +877,7 @@ sprixel_state(const sprixel* s, int y, int x){
   const ncplane* stdn = notcurses_stdplane_const(ncplane_notcurses_const(s->n));
   int localy = y - (s->n->absy - stdn->absy);
   int localx = x - (s->n->absx - stdn->absx);
-//fprintf(stderr, "TAM %d at %d/%d (%d/%d, %d/%d)\n", s->n->tam[localy * s->dimx + localx].state, localy, localx, y, x, s->dimy, s->dimx);
+//fprintf(stderr, "TAM %%d at %d/%d (%d/%d, %d/%d)\n", /*s->n->tam[localy * s->dimx + localx].state,*/ localy, localx, y, x, s->dimy, s->dimx);
   assert(localy >= 0);
   assert(localy < (int)s->dimy);
   assert(localx >= 0);
@@ -1075,10 +1073,12 @@ ALLOC char* ncplane_vprintf_prep(const char* format, va_list ap);
 
 // Resize the provided ncvisual to the specified 'rows' x 'cols', but do not
 // change the internals of the ncvisual. Uses oframe.
-int ncvisual_blit_internal(struct ncvisual* ncv, int rows, int cols,
+int ncvisual_blit_internal(const struct ncvisual* ncv, int rows, int cols,
                            ncplane* n, const struct blitset* bset,
                            const blitterargs* bargs);
 
+// if fd < 0, blocking_write() is going to emit an EBADF, so we don't
+// bother checking it here explicitly.
 static inline int
 tty_emit(const char* seq, int fd){
   if(!seq){
@@ -1333,43 +1333,57 @@ cell_set_fchannel(nccell* cl, uint32_t channel){
 }
 
 // Returns the result of blending two channels. 'blends' indicates how heavily
-// 'c1' ought be weighed. If 'blends' is 0, 'c1' will be entirely replaced by
-// 'c2'. If 'c1' is otherwise the default color, 'c1' will not be touched,
-// since we can't blend default colors. Likewise, if 'c2' is a default color,
-// it will not be used (unless 'blends' is 0).
+// 'c1' ought be weighed. If 'blends' is 0 (indicating that 'c1' has not yet
+// been set), 'c1' will be entirely determined by 'c2'. Otherwise, the default
+// color is preserved if both are the default, palette color is preserved if
+// both are the same palette index, and the result is otherwise RGB.
 static inline unsigned
-channels_blend(notcurses* nc, unsigned c1, unsigned c2, unsigned* blends){
+channels_blend(notcurses* nc, unsigned c1, unsigned c2, unsigned* blends,
+               uint32_t defchan){
   if(ncchannel_alpha(c2) == NCALPHA_TRANSPARENT){
     return c1; // do *not* increment *blends
   }
-  bool c2default = ncchannel_default_p(c2);
-  bool c2palette = ncchannel_palindex_p(c2);
   if(*blends == 0){
     // don't just return c2, or you set wide status and all kinds of crap
-    if(c2default){
+    if(ncchannel_default_p(c2)){
       ncchannel_set_default(&c1);
-    }else if(c2palette){
+    }else if(ncchannel_palindex_p(c2)){
       ncchannel_set_palindex(&c1, ncchannel_palindex(c2));
     }else{
-      ncchannel_set(&c1, c2 & NC_BG_RGB_MASK);
+      ncchannel_set(&c1, ncchannel_rgb(c2));
     }
-    ncchannel_set_alpha(&c1, ncchannel_alpha(c2));
-  }else if(!c2default && !ncchannel_default_p(c1)){
-    unsigned rsum, gsum, bsum;
-    if(c2palette){
-      uint32_t rgb = nc->palette.chans[ncchannel_palindex(c2)];
-      bsum = rgb & 0xff;
-      gsum = (rgb >> 8u) & 0xff;
-      rsum = (rgb >> 16u) & 0xff;
+  }else if(ncchannel_default_p(c1) && ncchannel_default_p(c2)){
+    // do nothing, leave as default
+  // intentional bitwise AND on the first condition, to eliminate the
+  // dependency due to C's short-circuit evaluation
+  }else if((ncchannel_palindex_p(c1) & ncchannel_palindex_p(c2)) &&
+           ncchannel_palindex(c1) == ncchannel_palindex(c2)){
+    // do nothing, leave as palette
+  }else{
+    unsigned r1, g1, b1;
+    unsigned r2, g2, b2;
+    if(ncchannel_default_p(c2)){
+      ncchannel_rgb8(defchan, &r2, &g2, &b2);
+    }else if(ncchannel_palindex_p(c2)){
+      ncchannel_rgb8(nc->palette.chans[ncchannel_palindex(c2)],
+                     &r2, &g2, &b2);
     }else{
-      ncchannel_rgb8(c2, &rsum, &gsum, &bsum);
+      ncchannel_rgb8(c2, &r2, &g2, &b2);
     }
-    rsum = (ncchannel_r(c1) * *blends + rsum) / (*blends + 1);
-    gsum = (ncchannel_g(c1) * *blends + gsum) / (*blends + 1);
-    bsum = (ncchannel_b(c1) * *blends + bsum) / (*blends + 1);
-    ncchannel_set_rgb8(&c1, rsum, gsum, bsum);
-    ncchannel_set_alpha(&c1, ncchannel_alpha(c2));
+    if(ncchannel_default_p(c1)){
+      ncchannel_rgb8(defchan, &r1, &g1, &b1);
+    }else if(ncchannel_palindex_p(c1)){
+      ncchannel_rgb8(nc->palette.chans[ncchannel_palindex(c1)],
+                     &r1, &g1, &b1);
+    }else{
+      ncchannel_rgb8(c1, &r1, &g1, &b1);
+    }
+    unsigned r = (r1 * *blends + r2) / (*blends + 1);
+    unsigned g = (g1 * *blends + g2) / (*blends + 1);
+    unsigned b = (b1 * *blends + b2) / (*blends + 1);
+    ncchannel_set_rgb8(&c1, r, g, b);
   }
+  ncchannel_set_alpha(&c1, ncchannel_alpha(c2));
   ++*blends;
   return c1;
 }
@@ -1377,12 +1391,14 @@ channels_blend(notcurses* nc, unsigned c1, unsigned c2, unsigned* blends){
 // do not pass palette-indexed channels!
 static inline uint64_t
 cell_blend_fchannel(notcurses* nc, nccell* cl, unsigned channel, unsigned* blends){
-  return cell_set_fchannel(cl, channels_blend(nc, cell_fchannel(cl), channel, blends));
+  return cell_set_fchannel(cl, channels_blend(nc, cell_fchannel(cl),
+                           channel, blends, nc->tcache.fg_default));
 }
 
 static inline uint64_t
 cell_blend_bchannel(notcurses* nc, nccell* cl, unsigned channel, unsigned* blends){
-  return cell_set_bchannel(cl, channels_blend(nc, cell_bchannel(cl), channel, blends));
+  return cell_set_bchannel(cl, channels_blend(nc, cell_bchannel(cl),
+                           channel, blends, nc->tcache.bg_collides_default));
 }
 
 // a sprixel occupies the entirety of its associated plane, usually an entirely
@@ -1438,7 +1454,7 @@ pool_blit_direct(egcpool* pool, nccell* c, const char* gcluster, int bytes, int
   // we allow newlines to be blitted into the pool, as they're picked up and
   // given special semantics by paint() and rasterization.
   if(*gcluster != '\n' && is_control_egc((const unsigned char*)gcluster, bytes)){
-    logerror("not loading control character %u\n", *(const unsigned char*)gcluster);
+    logerror("not loading control character %u", *(const unsigned char*)gcluster);
     return -1;
   }
   c->width = cols;
@@ -1533,14 +1549,14 @@ check_geometry_args(const ncplane* n, int y, int x,
   // handle the special -1 case for y/x, and reject other negatives
   if(y < 0){
     if(y != -1){
-      logerror("invalid y: %d\n", y);
+      logerror("invalid y: %d", y);
       return -1;
     }
     y = n->y;
   }
   if(x < 0){
     if(x != -1){
-      logerror("invalid x: %d\n", x);
+      logerror("invalid x: %d", x);
       return -1;
     }
     x = n->x;
@@ -1552,7 +1568,7 @@ check_geometry_args(const ncplane* n, int y, int x,
   unsigned ymax, xmax;
   ncplane_dim_yx(n, &ymax, &xmax);
   if(*ystart >= ymax || *xstart >= xmax){
-    logerror("invalid starting coordinates: %u/%u\n", *ystart, *xstart);
+    logerror("invalid starting coordinates: %u/%u", *ystart, *xstart);
     return -1;
   }
   // handle the special 0 case for ylen/xlen
@@ -1564,20 +1580,20 @@ check_geometry_args(const ncplane* n, int y, int x,
   }
   // ensure ylen/xlen are on-plane
   if(*ylen > ymax){
-    logerror("ylen > dimy %u > %u\n", *ylen, ymax);
+    logerror("ylen > dimy %u > %u", *ylen, ymax);
     return -1;
   }
   if(*xlen > xmax){
-    logerror("xlen > dimx %u > %u\n", *xlen, xmax);
+    logerror("xlen > dimx %u > %u", *xlen, xmax);
     return -1;
   }
   // ensure x + xlen and y + ylen are on-plane, without overflow
   if(ymax - *ylen < *ystart){
-    logerror("y + ylen > ymax %u + %u > %u\n", *ystart, *ylen, ymax);
+    logerror("y + ylen > ymax %u + %u > %u", *ystart, *ylen, ymax);
     return -1;
   }
   if(xmax - *xlen < *xstart){
-    logerror("x + xlen > xmax %u + %u > %u\n", *xstart, *xlen, xmax);
+    logerror("x + xlen > xmax %u + %u > %u", *xstart, *xlen, xmax);
     return -1;
   }
   return 0;
@@ -1776,8 +1792,8 @@ create_polyfill_op(int y, int x, struct topolyfill** stck){
 typedef struct ncvisual_implementation {
   int (*visual_init)(int loglevel);
   void (*visual_printbanner)(fbuf* f);
-  int (*visual_blit)(struct ncvisual* ncv, unsigned rows, unsigned cols, ncplane* n,
-                     const struct blitset* bset, const blitterargs* barg);
+  int (*visual_blit)(const struct ncvisual* ncv, unsigned rows, unsigned cols,
+                     ncplane* n, const struct blitset* bset, const blitterargs* barg);
   struct ncvisual* (*visual_create)(void);
   struct ncvisual* (*visual_from_file)(const char* fname);
   // ncv constructors other than ncvisual_from_file() need to set up the
@@ -1800,28 +1816,13 @@ typedef struct ncvisual_implementation {
 // populated by libnotcurses.so if linked with multimedia
 API extern ncvisual_implementation* visual_implementation;
 
-// prepend base with the Notcurses data directory as configured.
-static inline char*
-prefix_data(const char* base){
-  // need a byte for each of directory separator and nul terminator
-  const size_t dlen = strlen(NOTCURSES_SHARE);
-  size_t len = dlen + strlen(base) + 2;
-  char* path = (char*)malloc(len); // cast for C++ includers
-  if(path){
-    memcpy(path, NOTCURSES_SHARE, dlen);
-    path[dlen] = path_separator();
-    strcpy(path + dlen + 1, base);
-  }
-  return path;
-}
-
 // within unix, we can just use isatty(3). on windows, things work
 // differently. for a true Windows Terminal, we'll have HANDLE pointers
 // rather than file descriptors. in cygwin/msys2, isatty(3) always fails.
-// so for __MINGW64__, always return true. otherwise return isatty(fd).
+// so for __MINGW32__, always return true. otherwise return isatty(fd).
 static inline int
 tty_check(int fd){
-#ifdef __MINGW64__
+#ifdef __MINGW32__
   return _isatty(fd);
 #endif
   return isatty(fd);
@@ -1832,10 +1833,10 @@ tty_check(int fd){
 static inline int
 cancel_and_join(const char* name, pthread_t tid, void** res){
   if(pthread_cancel(tid)){
-    logerror("couldn't cancel %s thread\n", name); // tid might have died
+    logerror("couldn't cancel %s thread", name); // tid might have died
   }
   if(pthread_join(tid, res)){
-    logerror("error joining %s thread\n", name);
+    logerror("error joining %s thread", name);
     return -1;
   }
   return 0;
@@ -1843,7 +1844,7 @@ cancel_and_join(const char* name, pthread_t tid, void** res){
 
 static inline int
 emit_scrolls(const tinfo* ti, int count, fbuf* f){
-  logdebug("emitting %d scrolls\n", count);
+  logdebug("emitting %d scrolls", count);
   if(count > 1){
     const char* indn = get_escape(ti, ESCAPE_INDN);
     if(indn){
