@@ -92,10 +92,10 @@ dup_menu_item(ncmenu_int_item* dst, const struct ncmenu_item* src){
     return 0;
   }
   size_t bytes = 1; // NUL terminator
-  if(src->shortcut.alt){
+  if(ncinput_alt_p(&src->shortcut)){
     bytes += strlen(ALTMOD);
   }
-  if(src->shortcut.ctrl){
+  if(ncinput_ctrl_p(&src->shortcut)){
     bytes += strlen(CTLMOD);
   }
   mbstate_t ps;
@@ -107,8 +107,8 @@ dup_menu_item(ncmenu_int_item* dst, const struct ncmenu_item* src){
   }
   bytes += shortsize + 1;
   char* sdup = malloc(bytes);
-  int n = snprintf(sdup, bytes, "%s%s", src->shortcut.alt ? ALTMOD : "",
-                   src->shortcut.ctrl ? CTLMOD : "");
+  int n = snprintf(sdup, bytes, "%s%s", ncinput_alt_p(&src->shortcut) ? ALTMOD : "",
+                   ncinput_ctrl_p(&src->shortcut) ? CTLMOD : "");
   if(n < 0 || (size_t)n >= bytes){
     free(sdup);
     free(dst->desc);
@@ -375,16 +375,16 @@ resize_menu(ncplane* n){
 }
 
 ncmenu* ncmenu_create(ncplane* n, const ncmenu_options* opts){
-  ncmenu_options zeroed = {};
+  ncmenu_options zeroed = {0};
   if(!opts){
     opts = &zeroed;
   }
   if(opts->sectioncount <= 0 || !opts->sections){
-    logerror("Invalid %d-ary section information\n", opts->sectioncount);
+    logerror("invalid %d-ary section information", opts->sectioncount);
     return NULL;
   }
   if(opts->flags >= (NCMENU_OPTION_HIDING << 1u)){
-    logwarn("Provided unsupported flags %016" PRIx64 "\n", opts->flags);
+    logwarn("provided unsupported flags %016" PRIx64, opts->flags);
   }
   unsigned totalheight = 1;
   unsigned totalwidth = 2; // start with two-character margin on the left
@@ -435,7 +435,7 @@ ncmenu* ncmenu_create(ncplane* n, const ncmenu_options* opts){
     }
     free(ret);
   }
-  logerror("Error creating ncmenu\n");
+  logerror("error creating ncmenu");
   return NULL;
 }
 
@@ -454,7 +454,7 @@ int ncmenu_unroll(ncmenu* n, int sectionidx){
     return -1;
   }
   if(sectionidx < 0 || sectionidx >= n->sectioncount){
-    logerror("Unrolled invalid sectionidx %d\n", sectionidx);
+    logerror("unrolled invalid sectionidx %d", sectionidx);
     return -1;
   }
   if(n->sections[sectionidx].enabled_item_count <= 0){
