@@ -2,7 +2,6 @@
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <wctype.h>
 #include <getopt.h>
 #include <inttypes.h>
 #include <sys/mman.h>
@@ -11,16 +10,18 @@
 #include <notcurses/notcurses.h>
 #include "structure.h"
 #include "builddef.h"
+#include "parse.h"
 
 static void
 usage(const char* argv0, FILE* o){
-  fprintf(o, "usage: %s [ -hV ] files\n", argv0);
+  fprintf(o, "usage: %s [ -hVq ] files\n", argv0);
   fprintf(o, " -h: print help and return success\n");
   fprintf(o, " -v: print version and return success\n");
+  fprintf(o, " -q: don't wait for any input\n");
 }
 
 static int
-parse_args(int argc, char** argv){
+parse_args(int argc, char** argv, unsigned* noui){
   const char* argv0 = *argv;
   int longindex;
   int c;
@@ -28,7 +29,7 @@ parse_args(int argc, char** argv){
     { .name = "help", .has_arg = 0, .flag = NULL, .val = 'h', },
     { .name = NULL, .has_arg = 0, .flag = NULL, .val = 0, }
   };
-  while((c = getopt_long(argc, argv, "hV", longopts, &longindex)) != -1){
+  while((c = getopt_long(argc, argv, "hVq", longopts, &longindex)) != -1){
     switch(c){
       case 'h': usage(argv0, stdout);
                 exit(EXIT_SUCCESS);
@@ -36,6 +37,8 @@ parse_args(int argc, char** argv){
       case 'V': fprintf(stderr, "%s version %s\n", argv[0], notcurses_version());
                 exit(EXIT_SUCCESS);
                 break;
+      case 'q': *noui = true;
+                break;
       default: usage(argv0, stderr);
                return -1;
                break;
@@ -78,7 +81,7 @@ map_gzipped_data(unsigned char* buf, size_t* len, unsigned char* ubuf, uint32_t
 #include <zlib.h>
 static unsigned char*
 map_gzipped_data(unsigned char* buf, size_t* len, unsigned char* ubuf, uint32_t ulen){
-  z_stream z = {};
+  z_stream z = {0};
   int r = inflateInit2(&z, 15 | 16);
   if(r != Z_OK){
     fprintf(stderr, "error getting zlib inflator (%d)\n", r);
@@ -163,421 +166,6 @@ get_troff_data(const char *arg, size_t* len){
   return buf;
 }
 
-typedef enum {
-  LINE_UNKNOWN,
-  LINE_COMMENT,
-  LINE_B, LINE_BI, LINE_BR, LINE_I, LINE_IB, LINE_IR,
-  LINE_RB, LINE_RI, LINE_SB, LINE_SM,
-  LINE_EE, LINE_EX, LINE_RE, LINE_RS,
-  LINE_SH, LINE_SS, LINE_TH,
-  LINE_IP, LINE_LP, LINE_P, LINE_PP,
-  LINE_TP, LINE_TQ,
-  LINE_ME, LINE_MT, LINE_UE, LINE_UR,
-  LINE_OP, LINE_SY, LINE_YS,
-  LINE_NF, LINE_FI,
-} ltypes;
-
-typedef enum {
-  TROFF_UNKNOWN,
-  TROFF_COMMENT,
-  TROFF_FONT,
-  TROFF_STRUCTURE,
-  TROFF_PARAGRAPH,
-  TROFF_HYPERLINK,
-  TROFF_SYNOPSIS,
-  TROFF_PREFORMATTED,
-} ttypes;
-
-typedef struct {
-  ltypes ltype;
-  const char* symbol;
-  ttypes ttype;
-  uint32_t channel;
-} trofftype;
-
-// all troff types start with a period, followed by one or two ASCII
-// characters.
-static const trofftype trofftypes[] = {
-  { .ltype = LINE_UNKNOWN, .symbol = "", .ttype = TROFF_UNKNOWN, .channel = 0, },
-  { .ltype = LINE_COMMENT, .symbol = "\\\"", .ttype = TROFF_COMMENT, .channel = 0, },
-#define TROFF_FONT(x) { .ltype = LINE_##x, .symbol = #x, .ttype = TROFF_FONT, .channel = 0, },
-  TROFF_FONT(B) TROFF_FONT(BI) TROFF_FONT(BR)
-  TROFF_FONT(I) TROFF_FONT(IB) TROFF_FONT(IR)
-#undef TROFF_FONT
-#define TROFF_STRUCTURE(x, c) { .ltype = LINE_##x, .symbol = #x, .ttype = TROFF_STRUCTURE, .channel = (c), },
-  TROFF_STRUCTURE(EE, 0)
-  TROFF_STRUCTURE(EX, 0)
-  TROFF_STRUCTURE(RE, 0)
-  TROFF_STRUCTURE(RS, 0)
-  TROFF_STRUCTURE(SH, NCCHANNEL_INITIALIZER(0x9b, 0x9b, 0xfc))
-  TROFF_STRUCTURE(SS, NCCHANNEL_INITIALIZER(0x6c, 0x6b, 0xfb))
-  TROFF_STRUCTURE(TH, NCCHANNEL_INITIALIZER(0xcb, 0xcb, 0xfd))
-#undef TROFF_STRUCTURE
-#define TROFF_PARA(x) { .ltype = LINE_##x, .symbol = #x, .ttype = TROFF_PARAGRAPH, .channel = 0, },
-  TROFF_PARA(IP) TROFF_PARA(LP) TROFF_PARA(P)
-  TROFF_PARA(PP) TROFF_PARA(TP) TROFF_PARA(TQ)
-#undef TROFF_PARA
-#define TROFF_HLINK(x) { .ltype = LINE_##x, .symbol = #x, .ttype = TROFF_HYPERLINK, .channel = 0, },
-  TROFF_HLINK(ME) TROFF_HLINK(MT) TROFF_HLINK(UE) TROFF_HLINK(UR)
-#undef TROFF_HLINK
-#define TROFF_SYNOPSIS(x) { .ltype = LINE_##x, .symbol = #x, .ttype = TROFF_SYNOPSIS, .channel = 0, },
-  TROFF_SYNOPSIS(OP) TROFF_SYNOPSIS(SY) TROFF_SYNOPSIS(YS)
-#undef TROFF_SYNOPSIS
-  { .ltype = LINE_UNKNOWN, .symbol = "hy", .ttype = TROFF_UNKNOWN, .channel = 0, },
-  { .ltype = LINE_UNKNOWN, .symbol = "br", .ttype = TROFF_UNKNOWN, .channel = 0, },
-  { .ltype = LINE_COMMENT, .symbol = "IX", .ttype = TROFF_COMMENT, .channel = 0, },
-  { .ltype = LINE_NF, .symbol = "nf", .ttype = TROFF_FONT, .channel = 0, },
-  { .ltype = LINE_FI, .symbol = "fi", .ttype = TROFF_FONT, .channel = 0, },
-};
-
-// the troff trie is only defined on the 128 ascii values.
-struct troffnode {
-  struct troffnode* next[0x80];
-  const trofftype *ttype;
-};
-
-static void
-destroy_trofftrie(struct troffnode* root){
-  if(root){
-    for(unsigned i = 0 ; i < sizeof(root->next) / sizeof(*root->next) ; ++i){
-      destroy_trofftrie(root->next[i]);
-    }
-    free(root);
-  }
-}
-
-// build a trie rooted at an implicit leading period.
-static struct troffnode*
-trofftrie(void){
-  struct troffnode* root = malloc(sizeof(*root));
-  if(root == NULL){
-    return NULL;
-  }
-  memset(root, 0, sizeof(*root));
-  for(size_t toff = 0 ; toff < sizeof(trofftypes) / sizeof(*trofftypes) ; ++toff){
-    const trofftype* t = &trofftypes[toff];
-    if(strlen(t->symbol) == 0){
-      continue;
-    }
-    struct troffnode* n = root;
-    for(const char* s = t->symbol ; *s ; ++s){
-      if(*s < 0){ // illegal symbol
-        fprintf(stderr, "illegal symbol: %s\n", t->symbol);
-        goto err;
-      }
-      unsigned char us = *s;
-      if(us > sizeof(root->next) / sizeof(*root->next)){ // illegal symbol
-        fprintf(stderr, "illegal symbol: %s\n", t->symbol);
-        goto err;
-      }
-      if(n->next[us] == NULL){
-        if((n->next[us] = malloc(sizeof(*root))) == NULL){
-          goto err;
-        }
-        memset(n->next[us], 0, sizeof(*root));
-      }
-      n = n->next[us];
-    }
-    if(n->ttype){ // duplicate command
-      fprintf(stderr, "duplicate command: %s %s\n", t->symbol, n->ttype->symbol);
-      goto err;
-    }
-    n->ttype = t;
-  }
-  return root;
-
-err:
-  destroy_trofftrie(root);
-  return NULL;
-}
-
-// lex the troffnode out from |ws|, where the troffnode is all text prior to
-// whitespace or a NUL. the byte following the troffnode is written back to
-// |ws|. if it is a valid troff command sequence, the node is returned;
-// NULL is otherwise returned. |len| ought be non-negative.
-static const trofftype*
-get_type(const struct troffnode* trie, const unsigned char** ws, size_t len){
-  if(**ws != '.'){
-    return NULL;
-  }
-  ++*ws;
-  --len;
-  while(len && !isspace(**ws) && **ws){
-    if(**ws > sizeof(trie->next) / sizeof(*trie->next)){ // illegal command
-      return NULL;
-    }
-    if((trie = trie->next[**ws]) == NULL){
-      return NULL;
-    }
-    ++*ws;
-    --len;
-  }
-  return trie->ttype;
-}
-
-typedef struct pagenode {
-  char* text;
-  const trofftype* ttype;
-  struct pagenode* subs;
-  unsigned subcount;
-} pagenode;
-
-typedef struct pagedom {
-  struct pagenode* root;
-  struct troffnode* trie;
-  char* title;
-  char* section;
-  char* version;
-  char* footer;
-  char* header;
-  struct docstructure* ds;
-} pagedom;
-
-static const char*
-dom_get_title(const pagedom* dom){
-  return dom->title;
-}
-
-// get the next token. first, chew whitespace. then match a string of
-// iswgraph(), or a quoted string of iswprint(). return the number of
-// characters consumed, or -1 on error (no token, unterminated quote).
-// heap-copies the utf8 to *token on success.
-static int
-lex_next_token(const char* s, char** token){
-  mbstate_t ps = {};
-  wchar_t w;
-  size_t b, cur;
-  cur = 0;
-  bool inquote = false;
-  const char* tokstart = NULL;
-  while((b = mbrtowc(&w, s + cur, MB_CUR_MAX, &ps)) != (size_t)-1 && b != (size_t)-2){
-    if(tokstart){
-      if(b == 0 || (inquote && w == L'"') || (!inquote && iswspace(w))){
-        if(!tokstart || !*tokstart || *tokstart == '"'){
-          return -1;
-        }
-        *token = strndup(tokstart, cur - (tokstart - s));
-        return cur + b;
-      }
-    }else{
-      if(iswspace(w)){
-        cur += b;
-        continue;
-      }
-      if(w == '"'){
-        inquote = true;
-        tokstart = s + cur + b;
-      }else{
-        tokstart = s + cur;
-      }
-    }
-    cur += b;
-  }
-  return -1;
-}
-
-// take the newly-added title section, and extract the title, section, and
-// version (technically footer-middle, footer-inside, and header-middle).
-// they ought be quoted, but might not be.
-static int
-lex_title(pagedom* dom){
-  const char* tok = dom->root->text;
-  int b = lex_next_token(tok, &dom->title);
-  if(b < 0){
-    fprintf(stderr, "couldn't extract title [%s]\n", dom->root->text);
-    return -1;
-  }
-  tok += b;
-  b = lex_next_token(tok, &dom->section);
-  if(b < 0){
-    fprintf(stderr, "couldn't extract section [%s]\n", dom->root->text);
-    return -1;
-  }
-  tok += b;
-  b = lex_next_token(tok, &dom->version);
-  if(b < 0){
-    //fprintf(stderr, "couldn't extract version [%s]\n", dom->root->text);
-    return 0;
-  }
-  tok += b;
-  b = lex_next_token(tok, &dom->footer);
-  if(b < 0){
-    //fprintf(stderr, "couldn't extract footer [%s]\n", dom->root->text);
-    return 0;
-  }
-  tok += b;
-  b = lex_next_token(tok, &dom->header);
-  if(b < 0){
-    //fprintf(stderr, "couldn't extract header [%s]\n", dom->root->text);
-    return 0;
-  }
-  return 0;
-}
-
-static pagenode*
-add_node(pagenode* pnode, char* text){
-  unsigned ncount = pnode->subcount + 1;
-  pagenode* tmpsubs = realloc(pnode->subs, sizeof(*pnode->subs) * ncount);
-  if(tmpsubs == NULL){
-    return NULL;
-  }
-  pnode->subs = tmpsubs;
-  pagenode* r = pnode->subs + pnode->subcount;
-  pnode->subcount = ncount;
-  memset(r, 0, sizeof(*r));
-  r->text = text;
-//fprintf(stderr, "ADDED SECTION %s %u\n", text, pnode->subcount);
-  return r;
-}
-
-static char*
-extract_text(const unsigned char* ws, const unsigned char* feol){
-  if(ws == feol || ws == feol + 1){
-    fprintf(stderr, "bogus empty title\n");
-    return NULL;
-  }
-  return strndup((const char*)ws + 1, feol - ws);
-}
-
-static char*
-augment_text(pagenode* pnode, const unsigned char* ws, const unsigned char* feol){
-  const size_t slen = pnode->text ? strlen(pnode->text) + 1 : 0;
-  char* tmp = realloc(pnode->text, slen + (feol - ws) + 2);
-  if(tmp == NULL){
-    return NULL;
-  }
-  pnode->text = tmp;
-  if(slen){
-    pnode->text[slen - 1] = ' ';
-  }
-  memcpy(pnode->text + slen, ws, feol - ws + 1);
-  pnode->text[slen + (feol - ws + 1)] = '\0';
-  return pnode->text;
-}
-
-// extract the page structure.
-// FIXME we need to fuzz this, hard
-static int
-troff_parse(const unsigned char* map, size_t mlen, pagedom* dom){
-  const struct troffnode* trie = dom->trie;
-  const unsigned char* line = map;
-  pagenode* current_section = NULL;
-  pagenode* current_subsection = NULL;
-  pagenode* current_para = NULL;
-  bool preformatted = false;
-  for(size_t off = 0 ; off < mlen ; ++off){
-    const unsigned char* ws = line;
-    size_t left = mlen - off;
-    const trofftype* node = get_type(trie, &ws, left);
-    // find the end of this line
-    const unsigned char* eol = ws;
-    left -= (ws - line);
-    while(left && *eol != '\n' && *eol){
-      ++eol;
-      --left;
-    }
-    const unsigned char* feol = eol;
-    // functional end of line--doesn't include possible newline
-    if(!preformatted){
-      if(left && *eol == '\n'){
-        --feol;
-      }
-    }
-    if(node == NULL){
-      if(current_para == NULL){
-        //fprintf(stderr, "free-floating text transcends para\n");
-        //fprintf(stderr, "[%s]\n", line);
-      }else{
-        char* et = augment_text(current_para, line, feol);
-        if(et == NULL){
-          return -1;
-        }
-      }
-    }else if(node->ltype == LINE_NF){
-      preformatted = true;
-    }else if(node->ltype == LINE_FI){
-      preformatted = false;
-    }else if(node->ltype == LINE_TH){
-      if(dom_get_title(dom)){
-        fprintf(stderr, "found a second title (was %s)\n", dom_get_title(dom));
-        return -1;
-      }
-      char* et = extract_text(ws, feol);
-      if(et == NULL){
-        return -1;
-      }
-      if((dom->root = malloc(sizeof(*dom->root))) == NULL){
-        free(et);
-        return -1;
-      }
-      memset(dom->root, 0, sizeof(*dom->root));
-      dom->root->ttype = node;
-      dom->root->text = et;
-      if(lex_title(dom)){
-        return -1;
-      }
-      current_para = dom->root;
-    }else if(node->ltype == LINE_SH){
-      if(dom->root == NULL){
-        fprintf(stderr, "section transcends structure\n");
-        return -1;
-      }
-      char* et = extract_text(ws, feol);
-      if(et == NULL){
-        return -1;
-      }
-      if((current_section = add_node(dom->root, et)) == NULL){
-        free(et);
-        return -1;
-      }
-      current_section->ttype = node;
-      current_subsection = NULL;
-      current_para = current_section;
-    }else if(node->ltype == LINE_SS){
-      char* et = extract_text(ws, feol);
-      if(et == NULL){
-        return -1;
-      }
-      if(current_section == NULL){
-        fprintf(stderr, "subsection %s without section\n", et);
-        free(et);
-        return -1;
-      }
-      if((current_subsection = add_node(current_section, et)) == NULL){
-        free(et);
-        return -1;
-      }
-      current_subsection->ttype = node;
-      current_para = current_subsection;
-    }else if(node->ltype == LINE_PP){
-      if(dom->root == NULL){
-        fprintf(stderr, "paragraph transcends structure\n");
-        return -1;
-      }
-      if((current_para = add_node(current_para, NULL)) == NULL){
-        return -1;
-      }
-      current_para->ttype = node;
-    }else if(node->ltype == LINE_TP){
-      if(dom->root == NULL){
-        fprintf(stderr, "tagged paragraph transcends structure\n");
-        return -1;
-      }
-      if((current_para = add_node(current_para, NULL)) == NULL){
-        return -1;
-      }
-      current_para->ttype = node;
-    }
-    off += eol - line;
-    line = eol + 1;
-  }
-  if(dom_get_title(dom) == NULL){
-    fprintf(stderr, "no title found\n");
-    return -1;
-  }
-  return 0;
-}
-
 // invoke ncplane_puttext() on the text starting at s and ending
 // (non-inclusive) at e.
 static int
@@ -596,6 +184,22 @@ puttext(struct ncplane* p, const char* s, const char* e){
   return r;
 }
 
+// try to identify a unicode macro in |macro|, and if so, update |end|. returns
+// a static buffer, so this is not threadsafe/reentrant.
+static const char*
+unicode_macro(const char* macro, const char** end){
+  unsigned long u;
+  u = strtoul(macro + 1, (char**)end, 16);
+  if(**end != ']'){
+    return NULL;
+  }
+  ++*end;
+  static char fill[5];
+  wint_t wu = u;
+  snprintf(fill, sizeof(fill), "%lc", wu);
+  return fill;
+}
+
 // paragraphs can have formatting information inline within their text. we
 // proceed until we find such an inline marker, print the text we've skipped,
 // set up the style, and continue.
@@ -673,8 +277,11 @@ putpara(struct ncplane* p, const char* text){
             }
           }
           if(macend == NULL){
-            fprintf(stderr, "unknown macro %s\n", curend);
-            return -1;
+            posttext = unicode_macro(curend, &macend);
+            if(posttext == NULL){
+              fprintf(stderr, "unknown macro %s\n", curend);
+              return -1;
+            }
           }
           curend = macend;
           break;
@@ -707,12 +314,14 @@ putpara(struct ncplane* p, const char* text){
 
 static int
 draw_domnode(struct ncplane* p, const pagedom* dom, const pagenode* n,
-             unsigned* wrotetext){
+             unsigned* wrotetext, unsigned* insubsec){
   ncplane_set_fchannel(p, n->ttype->channel);
   size_t b = 0;
+  unsigned y;
+  ncplane_cursor_yx(p, &y, NULL);
   switch(n->ttype->ltype){
     case LINE_TH:
-      if(docstructure_add(dom->ds, dom->title, ncplane_y(p), DOCSTRUCTURE_TITLE)){
+      if(docstructure_add(dom->ds, dom->title, y)){
         return -1;
       }
       /*
@@ -722,7 +331,7 @@ draw_domnode(struct ncplane* p, const pagedom* dom, const pagenode* n,
       ncplane_set_styles(p, NCSTYLE_NONE);
       */break;
     case LINE_SH: // section heading
-      if(docstructure_add(dom->ds, dom->title, ncplane_y(p), DOCSTRUCTURE_SECTION)){
+      if(docstructure_add(dom->ds, n->text, y)){
         return -1;
       }
       if(strcmp(n->text, "NAME")){
@@ -735,7 +344,7 @@ draw_domnode(struct ncplane* p, const pagedom* dom, const pagenode* n,
       }
       break;
     case LINE_SS: // subsection heading
-      if(docstructure_add(dom->ds, dom->title, ncplane_y(p), DOCSTRUCTURE_SUBSECTION)){
+      if(docstructure_add(dom->ds, n->text, y)){
         return -1;
       }
       ncplane_puttext(p, -1, NCALIGN_LEFT, "\n\n", &b);
@@ -744,21 +353,31 @@ draw_domnode(struct ncplane* p, const pagedom* dom, const pagenode* n,
       ncplane_set_styles(p, NCSTYLE_NONE);
       ncplane_cursor_move_yx(p, -1, 0);
       *wrotetext = true;
+      *insubsec = true;
       break;
     case LINE_PP: // paragraph
     case LINE_TP: // tagged paragraph
     case LINE_IP: // indented paragraph
       if(*wrotetext){
         if(n->text){
-          ncplane_puttext(p, -1, NCALIGN_LEFT, "\n\n", &b);
+          ncplane_set_fg_rgb(p, 0xe0f0ff);
+          if(*insubsec){
+            ncplane_puttext(p, -1, NCALIGN_LEFT, "\n", &b);
+            *insubsec = false;
+          }else{
+            ncplane_puttext(p, -1, NCALIGN_LEFT, "\n\n", &b);
+          }
+          ncplane_set_fg_alpha(p, NCALPHA_HIGHCONTRAST);
           putpara(p, n->text);
         }
       }else{
-        ncplane_set_styles(p, NCSTYLE_BOLD | NCSTYLE_ITALIC);
-        ncplane_set_fg_rgb(p, 0xff6a00);
-        ncplane_putstr_aligned(p, -1, NCALIGN_CENTER, n->text);
-        ncplane_set_fg_default(p);
-        ncplane_set_styles(p, NCSTYLE_NONE);
+        if(n->text){
+          ncplane_set_styles(p, NCSTYLE_BOLD | NCSTYLE_ITALIC);
+          ncplane_set_fg_rgb(p, 0xff6a00);
+          ncplane_putstr_aligned(p, -1, NCALIGN_CENTER, n->text);
+          ncplane_set_fg_default(p);
+          ncplane_set_styles(p, NCSTYLE_NONE);
+        }
       }
       *wrotetext = true;
       break;
@@ -767,7 +386,7 @@ draw_domnode(struct ncplane* p, const pagedom* dom, const pagenode* n,
       return 0; // FIXME
   }
   for(unsigned z = 0 ; z < n->subcount ; ++z){
-    if(draw_domnode(p, dom, &n->subs[z], wrotetext)){
+    if(draw_domnode(p, dom, &n->subs[z], wrotetext, insubsec)){
       return -1;
     }
   }
@@ -781,11 +400,13 @@ static int
 draw_content(struct ncplane* stdn, struct ncplane* p){
   pagedom* dom = ncplane_userptr(p);
   unsigned wrotetext = 0; // unused by us
+  unsigned insubsec = 0;
+  docstructure_free(dom->ds);
   dom->ds = docstructure_create(stdn);
   if(dom->ds == NULL){
     return -1;
   }
-  return draw_domnode(p, dom, dom->root, &wrotetext);
+  return draw_domnode(p, dom, dom->root, &wrotetext, &insubsec);
 }
 
 static int
@@ -793,6 +414,7 @@ resize_pman(struct ncplane* pman){
   unsigned dimy, dimx;
   ncplane_dim_yx(ncplane_parent_const(pman), &dimy, &dimx);
   ncplane_resize_simple(pman, dimy - 1, dimx);
+  ncplane_erase(pman);
   struct ncplane* stdn = notcurses_stdplane(ncplane_notcurses(pman));
   int r = draw_content(stdn, pman);
   ncplane_move_yx(pman, 0, 0);
@@ -812,8 +434,9 @@ render_troff(struct notcurses* nc, const unsigned char* map, size_t mlen,
   }
   // this is just an estimate
   struct ncplane_options popts = {
-    .rows = dimy - 1,
+    .rows = dimy - 2,
     .cols = dimx,
+    .y = 1,
     .userptr = dom,
     .resizecb = resize_pman,
     .flags = NCPLANE_OPTION_AUTOGROW | NCPLANE_OPTION_VSCROLL,
@@ -822,6 +445,7 @@ render_troff(struct notcurses* nc, const unsigned char* map, size_t mlen,
   if(pman == NULL){
     return NULL;
   }
+  ncplane_set_base(pman, " ", 0, 0);
   if(draw_content(stdn, pman)){
     ncplane_destroy(pman);
     return NULL;
@@ -829,8 +453,8 @@ render_troff(struct notcurses* nc, const unsigned char* map, size_t mlen,
   return pman;
 }
 
-static const char USAGE_TEXT[] = "⎥b⇞k↑j↓f⇟⎢ (q)uit";
-static const char USAGE_TEXT_ASCII[] = "(bkjf) (q)uit";
+static const char USAGE_TEXT[] = "⎥h←s→l⎢⎥b⇞k↑↓j⇟f⎢ (q)uit";
+static const char USAGE_TEXT_ASCII[] = "(hsl) (bkjf) (q)uit";
 
 static int
 draw_bar(struct ncplane* bar, pagedom* dom){
@@ -857,6 +481,7 @@ resize_bar(struct ncplane* bar){
   unsigned dimy, dimx;
   ncplane_dim_yx(ncplane_parent_const(bar), &dimy, &dimx);
   ncplane_resize_simple(bar, 1, dimx);
+  ncplane_erase(bar);
   int r = draw_bar(bar, ncplane_userptr(bar));
   ncplane_move_yx(bar, dimy - 1, 0);
   return r;
@@ -918,12 +543,12 @@ create_bar(struct notcurses* nc, pagedom* dom){
 }
 
 static int
-manloop(struct notcurses* nc, const char* arg){
+manloop(struct notcurses* nc, const char* arg, unsigned noui){
   struct ncplane* stdn = notcurses_stdplane(nc);
   int ret = -1;
   struct ncplane* page = NULL;
   struct ncplane* bar = NULL;
-  pagedom dom = {};
+  pagedom dom = {0};
   size_t len;
   unsigned char* buf = get_troff_data(arg, &len);
   if(buf == NULL){
@@ -941,11 +566,17 @@ manloop(struct notcurses* nc, const char* arg){
   if(bar == NULL){
     goto done;
   }
+  if(notcurses_render(nc)){
+    goto done;
+  }
+  if(noui){
+    ret = 0;
+    goto done;
+  }
   uint32_t key;
   do{
-    if(notcurses_render(nc)){
-      goto done;
-    }
+    bool movedown = false;
+    int newy = ncplane_y(page);
     ncinput ni;
     key = notcurses_get(nc, NULL, &ni);
     if(ni.evtype == NCTYPE_RELEASE){
@@ -957,35 +588,48 @@ manloop(struct notcurses* nc, const char* arg){
           notcurses_refresh(nc, NULL, NULL);
         }
         break;
+      case 's':
+        docstructure_toggle(page, bar, dom.ds);
+        break;
+      case 'h': case NCKEY_LEFT:
+        newy = docstructure_prev(dom.ds);
+        break;
+      case 'l': case NCKEY_RIGHT:
+        newy = docstructure_next(dom.ds);
+        movedown = true;
+        break;
       case 'k': case NCKEY_UP:
-        if(ncplane_y(page)){
-          ncplane_move_rel(page, 1, 0);
-        }
+        newy = ncplane_y(page) + 1;
         break;
       // we can move down iff our last line is beyond the visible area
       case 'j': case NCKEY_DOWN:
-        if(ncplane_y(page) + ncplane_dim_y(page) >= ncplane_dim_y(stdn)){
-          ncplane_move_rel(page, -1, 0);
-        }
+        newy = ncplane_y(page) - 1;
+        movedown = true;
         break;
       case 'b': case NCKEY_PGUP:{
-        int newy = ncplane_y(page) + (int)ncplane_dim_y(stdn);
-        if(newy > 0){
-          newy = 0;
-        }
-        ncplane_move_yx(page, newy, 0);
+        newy = ncplane_y(page) + (int)ncplane_dim_y(stdn);
         break;
       }case 'f': case NCKEY_PGDOWN:{
-        int newy = ncplane_y(page) - (int)ncplane_dim_y(stdn) + 1;
-        if(newy + (int)ncplane_dim_y(page) < (int)ncplane_dim_y(stdn)){
-          newy += (int)ncplane_dim_y(stdn) - (newy + (int)ncplane_dim_y(page)) - 1;
-        }
-        ncplane_move_yx(page, newy, 0);
+        newy = ncplane_y(page) - (int)ncplane_dim_y(stdn) + 1;
+        movedown = true;
         break;
       }case 'q':
         ret = 0;
         goto done;
     }
+    if(newy > 1){
+      newy = 1;
+    }
+    if(newy + (int)ncplane_dim_y(page) < (int)ncplane_dim_y(stdn)){
+      newy += (int)ncplane_dim_y(stdn) - (newy + (int)ncplane_dim_y(page)) - 1;
+    }
+    if(newy != ncplane_y(page)){
+      ncplane_move_yx(page, newy, 0);
+      docstructure_move(dom.ds, newy, movedown);
+      if(notcurses_render(nc)){
+        goto done;
+      }
+    }
   }while(key != (uint32_t)-1);
 
 done:
@@ -1001,18 +645,24 @@ done:
 }
 
 static int
-tfman(struct notcurses* nc, const char* arg){
-  int r = manloop(nc, arg);
+tfman(struct notcurses* nc, const char* arg, unsigned noui){
+  int r = manloop(nc, arg, noui);
   return r;
 }
 
 int main(int argc, char** argv){
-  int nonopt = parse_args(argc, argv);
+  unsigned noui = false;
+  int nonopt = parse_args(argc, argv, &noui);
   if(nonopt <= 0){
     return EXIT_FAILURE;
   }
-  struct notcurses_options nopts = {
-  };
+  struct notcurses_options nopts = {0};
+  if(noui){
+    nopts.flags |= NCOPTION_NO_ALTERNATE_SCREEN
+                   | NCOPTION_NO_CLEAR_BITMAPS
+                   | NCOPTION_PRESERVE_CURSOR
+                   | NCOPTION_DRAIN_INPUT;
+  }
   struct notcurses* nc = notcurses_core_init(&nopts, NULL);
   if(nc == NULL){
     return EXIT_FAILURE;
@@ -1020,7 +670,7 @@ int main(int argc, char** argv){
   bool success;
   for(int i = 0 ; i < argc - nonopt ; ++i){
     success = false;
-    if(tfman(nc, argv[nonopt + i])){
+    if(tfman(nc, argv[nonopt + i], noui)){
       break;
     }
     success = true;
